﻿using System.Web.UI;
using System.Web.Services;
using CodeEffects.Rule.Core;
using CodeEffects.Rule.Common;
using CodeEffects.Rule.Asp;
using CodeEffects.Rule.Demo.Asp.Common.Data;

namespace CodeEffects.Rule.Demo.Asp
{
	public partial class Ajax : Page
	{
		[WebMethod(false)]
		public static string LoadSettings(ThemeType theme, bool showHelpString, bool showLineDots)
		{
			RuleEditor ce = GetControl();

			ce.ShowLineDots = showLineDots;
			ce.ShowHelpString = showHelpString;
			ce.Theme = theme;

			// Load all your rules into the tool bar's rules menu. You need these rules there
			// in order to be able to load, edit and delete them without writing anything extra.
			ce.ToolBarRules = Storage.GetAllRules();
			if (ce.ToolBarRules.Count > 0) Storage.SortRules(ce.ToolBarRules);

			// Load only evaluation rules into context menus. No need to sort them - Web Rule will
			// mix these rules with rule fields and sort the whole list
			ce.ContextMenuRules = Storage.GetEvaluationRules();

			return ce.GetClientSettings();
		}

		[WebMethod(false)]
		public static string LoadRule(string ruleId)
		{
			RuleEditor ce = GetControl();
			ce.LoadRuleXml(Storage.LoadRuleXml(ruleId));
			return ce.GetClientRuleData();
		}

		[WebMethod(false)]
		public static void DeleteRule(string ruleId)
		{
			Storage.DeleteRule(ruleId);
		}

		[WebMethod(false)]
		public static ProcessingResult SaveRule(string ruleData)
		{
			ProcessingResult r = new ProcessingResult();
			RuleEditor ce = GetControl();
			ce.LoadClientData(ruleData);
			if (ce.IsEmpty)
			{
				r.IsRuleEmpty = true;
			}
			else if (!ce.IsValid)
			{
				r.IsRuleValid = false;
				r.ClientInvalidData = ce.GetClientInvalidData();
			}
			else
			{
				SaveEventArgs e = ce.GetSaveArguments();
				Storage.SaveRule(e);

				r.Output = e.RuleID.ToString();
			}
			return r;
		}

		[WebMethod(false)]
		public static ProcessingResult EvaluateRule(Patient patient, string ruleData)
		{
			ProcessingResult r = new ProcessingResult();

			// Get Web Rule instance
			RuleEditor ce = GetControl();

			// Tell Web Rule not to enforce the rule name requirement -
			// we don't save the rule, just evaluate it, so we don't care about its name
			ce.RuleNameIsRequired = false;

			// Load the rule data from the client
			ce.LoadClientData(ruleData);

			if (ce.IsEmpty)
			{
				r.IsRuleEmpty = true;
			}
			else if (!ce.IsValid)
			{
				// The rule is invalid. Load the invalid data and pass it to the client
				r.IsRuleValid = false;
				r.ClientInvalidData = ce.GetClientInvalidData();
			}
			else
			{
				// THe rule is fine. Evaluate it.
				// The parameter Storage.LoadRuleXml is a delegate that Web Rule
				// needs for looking up reusable rules that this rule might reference.
				Evaluator<Patient> e = new Evaluator<Patient>(ce.GetRuleXml(), Storage.LoadRuleXml);
				bool success = e.Evaluate(patient);

				r.Output = string.IsNullOrWhiteSpace(patient.Output) ? "The rule evaluated to " + success.ToString() : patient.Output;
			}
			return r;
		}

		private static RuleEditor GetControl()
		{
			RuleEditor ce = new RuleEditor();

			// The iD must be the same as in HTML
			ce.ID = "ruleControl";
			ce.Mode = RuleType.Execution;
			ce.ClientOnly = true;

			// This is where we specify the source object
			ce.SourceAssembly = "CodeEffects.Rule.Demo.Asp";
			ce.SourceType = "CodeEffects.Rule.Demo.Asp.Common.Data.Patient";

			return ce;
		}
	}
}